@@ -1,5 +1,5 @@
 /* cfg_driver.c -- Main driver for Mailutils configuration files
-   Copyright (C) 2007-2012 Free Software Foundation, Inc.
+   Copyright (C) 2007-2012, 2014-2016 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -27,7 +27,6 @@
 #include <ctype.h>
 #include <mailutils/argcv.h>
 #include <mailutils/nls.h>
-#define MU_CFG_COMPATIBILITY /* This source uses deprecated cfg interfaces */
 #include <mailutils/cfg.h>
 #include <mailutils/errno.h>
 #include <mailutils/error.h>
@@ -39,7 +38,7 @@
 #include <mailutils/stream.h>
 #include <mailutils/assoc.h>
 #include <mailutils/alloc.h>
-
+#include <mailutils/cstr.h>
 
 static mu_assoc_t section_tab;
 
@@ -463,27 +462,23 @@ mu_cfg_tree_reduce (mu_cfg_tree_t *parse_tree,
 		    void *target_ptr)
 {
   int rc = 0;
-
+  struct mu_cfg_cont *cont;
   if (!parse_tree)
     return 0;
-  if (hints && (hints->flags & MU_PARSE_CONFIG_DUMP))
+  if (hints && (hints->flags & MU_CF_DUMP))
     {
       int yes = 1;
       mu_stream_t stream;
       
       mu_stdio_stream_create (&stream, MU_STDERR_FD, 0);
       mu_stream_ioctl (stream, MU_IOCTL_FD, MU_IOCTL_FD_SET_BORROW, &yes);
-      mu_cfg_format_parse_tree (stream, parse_tree, MU_CFG_FMT_LOCUS);
+      mu_cfg_format_parse_tree (stream, parse_tree, MU_CF_FMT_LOCUS);
       mu_stream_destroy (&stream);
     }
 
-  if (root_container)
-    {
-      struct mu_cfg_cont *cont = mu_build_container (progparam);
-      rc = mu_cfg_scan_tree (parse_tree, &cont->v.section, target_ptr,
-			     NULL);
-      mu_config_destroy_container (&cont);
-    }
+  cont = mu_build_container (progparam);
+  rc = mu_cfg_scan_tree (parse_tree, &cont->v.section, target_ptr, NULL);
+  mu_config_destroy_container (&cont);
 
   return rc;
 }
@@ -496,29 +491,6 @@ mu_format_config_tree (mu_stream_t stream, struct mu_cfg_param *progparam)
   mu_config_destroy_container (&cont);
 }
 
-int
-mu_parse_config (const char *file, const char *progname,
-		 struct mu_cfg_param *progparam, int flags,
-		 void *target_ptr)
-{
-  int rc;
-  char *full_name = mu_tilde_expansion (file, MU_HIERARCHY_DELIMITER, NULL);
-  if (full_name)
-    {
-      if (access (full_name, R_OK) == 0)
-	{
-	  rc = mu_get_config (full_name, progname, progparam, flags,
-			      target_ptr);
-	}
-      else
-	rc = ENOENT;
-      free (full_name);
-    }
-  else
-    rc = ENOMEM;
-  return rc;
-}
-
 static const char *
 _first_value_ptr (mu_config_value_t *val)
 {
@@ -615,3 +587,92 @@ mu_cfg_string_value_cb (mu_config_value_t *val,
     }
   return rc;
 }
+
+struct mapping_closure
+{
+  mu_assoc_t assoc;
+  char *err_term;
+  int err;
+};
+
+static int
+parse_mapping (void *item, void *data)
+{
+  struct mapping_closure *clos = data;
+  char *str = item;
+  size_t len;
+  char *key, *val;
+  
+  len = strcspn (str, "=");
+  if (str[len] == 0)
+    {
+      clos->err_term = mu_strdup (str);
+      return MU_ERR_PARSE;
+    }
+  key = mu_alloc (len + 1);
+  memcpy (key, str, len);
+  key[len] = 0;
+  val = mu_strdup (str + len + 1);
+  if (!val)
+    return ENOMEM;
+  clos->err = mu_assoc_install (clos->assoc, key, &val);
+  free (key);
+  if (clos->err)
+    return 1;
+  return 0;
+}
+
+static void
+assoc_str_free (void *data)
+{
+  free (data);
+}
+
+int
+mu_cfg_field_map (struct mu_config_value const *val, mu_assoc_t *passoc,
+		  char **err_term)
+{
+  int rc;
+  struct mapping_closure clos;
+  mu_list_t list = NULL;
+  
+  rc = mu_assoc_create (&clos.assoc, sizeof(char*), 0);
+  if (rc)
+    return rc;
+  mu_assoc_set_free (clos.assoc, assoc_str_free);
+  clos.err_term = NULL;
+  
+  switch (val->type)
+    {
+    case MU_CFG_STRING:
+      mu_list_create (&list);
+      mu_list_set_destroy_item (list, mu_list_free_item);
+      rc = mu_string_split (val->v.string, ":", list);
+      if (rc == 0)
+	rc = mu_list_foreach (list, parse_mapping, &clos);
+      mu_list_destroy (&list);
+      break;
+
+    case MU_CFG_LIST:
+      rc = mu_list_foreach (val->v.list, parse_mapping, &clos);
+      break;
+
+    case MU_CFG_ARRAY:
+      rc = EINVAL;
+    }
+
+  if (rc)
+    {
+      if (err_term)
+	*err_term = clos.err_term;
+      else
+	free (clos.err_term);
+      mu_assoc_destroy (&clos.assoc);
+    }
+  else
+    *passoc = clos.assoc;
+  
+  return rc;
+}
+
+
