@@ -1,24 +1,21 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+/* A Bison parser, made by GNU Bison 2.7.  */
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+/* Bison interface for Yacc-like parsers in C
+   
+      Copyright (C) 1984, 1989-1990, 2000-2012 Free Software Foundation, Inc.
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,10 +26,20 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_YY_Y_TAB_H_INCLUDED
+# define YY_YY_Y_TAB_H_INCLUDED
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 1
+#endif
+#if YYDEBUG
+extern int mu_sieve_yydebug;
+#endif
+
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
@@ -50,7 +57,9 @@
      ELSE = 266,
      ANYOF = 267,
      ALLOF = 268,
-     NOT = 269
+     NOT = 269,
+     FALSE = 270,
+     TRUE = 271
    };
 #endif
 /* Tokens.  */
@@ -66,41 +75,69 @@
 #define ANYOF 267
 #define ALLOF 268
 #define NOT 269
-
+#define FALSE 270
+#define TRUE 271
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 33 "sieve.y"
 {
+/* Line 2058 of yacc.c  */
+#line 78 "sieve.y"
+
   char *string;
   size_t number;
-  sieve_instr_t instr;
-  mu_sieve_value_t *value;
-  mu_list_t list;
-  size_t pc;
-  struct {
-    size_t start;
-    size_t end;
-  } pclist;
-  struct {
+  size_t idx;
+  struct mu_sieve_slice slice;
+  struct
+  {
     char *ident;
-    mu_list_t args;
+    size_t first;
+    size_t count;
   } command;
-  struct {
-    size_t begin;
-    size_t cond;
-    size_t branch;
-  } branch;
-}
-/* Line 1489 of yacc.c.  */
-#line 99 "sieve-gram.h"
-	YYSTYPE;
+  struct node_list
+  {
+    struct mu_sieve_node *head, *tail;
+  } node_list;
+  struct mu_sieve_node *node;
+
+
+/* Line 2058 of yacc.c  */
+#line 108 "sieve-gram.h"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define mu_sieve_yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
+#endif
+
+#if ! defined YYLTYPE && ! defined YYLTYPE_IS_DECLARED
+typedef struct YYLTYPE
+{
+  int first_line;
+  int first_column;
+  int last_line;
+  int last_column;
+} YYLTYPE;
+# define mu_sieve_yyltype YYLTYPE /* obsolescent; will be withdrawn */
+# define YYLTYPE_IS_DECLARED 1
+# define YYLTYPE_IS_TRIVIAL 1
 #endif
 
 extern YYSTYPE mu_sieve_yylval;
+extern YYLTYPE mu_sieve_yylloc;
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int mu_sieve_yyparse (void *YYPARSE_PARAM);
+#else
+int mu_sieve_yyparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int mu_sieve_yyparse (void);
+#else
+int mu_sieve_yyparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
 
+#endif /* !YY_YY_Y_TAB_H_INCLUDED  */
