@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2007-2012, 2014-2015 Free Software Foundation, Inc.
+   Copyright (C) 2007-2012, 2014-2016 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -91,8 +91,6 @@
 # define USE_MAILBOX_QUOTAS 1
 #endif
 
-#include "mailutils/libargp.h"
-
 #include "tcpwrap.h"
 #include "muscript.h"
 
@@ -127,7 +125,8 @@ extern char *quota_query;
 extern char *forward_file;
 extern int forward_file_checks;
 
-extern char *sender_address;       
+extern char *sender_address;
+extern char *default_domain;
 extern mu_list_t script_list;
 extern char *message_id_header;
 extern int sieve_debug_flags; 
