@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2012, 2014-2015 Free Software Foundation, Inc.
+   Copyright (C) 2010-2012, 2014-2016 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,28 +14,11 @@
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
-#if defined(HAVE_CONFIG_H)
-# include <config.h>
-#endif
-#include <stdlib.h>
-#include <mailutils/mailutils.h>
-#include <mailutils/libcfg.h>
-#include <argp.h>
+#include "mu.h"
 
-static char ldflags_doc[] = N_("mu ldflags - list libraries required to link");
 char ldflags_docstring[] = N_("list libraries required to link");
 static char ldflags_args_doc[] = N_("KEYWORD [KEYWORD...]");
 
-static struct argp ldflags_argp = {
-  NULL,
-  NULL,
-  ldflags_args_doc,
-  ldflags_doc,
-  NULL,
-  NULL,
-  NULL
-};
-
 
 #ifdef WITH_TLS
 # define NEEDAUTH "-lmu_auth " AUTHLIBS
@@ -74,7 +57,6 @@ struct lib_descr {
 #endif
   { "mailer", "-lmu_mailer", 0 },
   { "sieve",  "-lmu_sieve",  0, NOTALL },
-  { "compat", "-lmu_compat", 0 },
   { "auth",   "-lmu_auth " AUTHLIBS, 2 },
 #ifdef WITH_GUILE	      
   { "guile",  "-lmu_scm " GUILE_LIBS, -1, NOTALL },
@@ -82,8 +64,9 @@ struct lib_descr {
 #ifdef WITH_PYTHON
   { "python", "-lmu_py " PYTHON_LIBS, -1, NOTALL },
 #endif
-  { "cfg",    "-lmu_cfg",  -1, NOTALL },
-  { "argp",   "-lmu_argp", -2, NOTALL },
+  { "compat", NULL,  0, 0 },  /* deprecated */
+  { "cfg",    NULL,  0, 0 },  /* deprecated */
+  { "argp",   NULL,  0, 0 },  /* deprecated */
   { NULL }
 };
 
@@ -117,7 +100,7 @@ add_entry (int level, char *ptr)
 
 /* Sort the entries by their level. */
 void
-sort_entries ()
+sort_entries (void)
 {
   int j;
 
@@ -140,13 +123,9 @@ sort_entries ()
 int
 mutool_ldflags (int argc, char **argv)
 {
-  int i, j;
-  
-  if (argp_parse (&ldflags_argp, argc, argv, ARGP_IN_ORDER, &i, NULL))
-    return 1;
+  int j;
 
-  argc -= i;
-  argv += i;
+  mu_action_getopt (&argc, &argv, NULL, ldflags_docstring, ldflags_args_doc);
   
   add_entry (-100, LINK_FLAGS);
   add_entry (100, LINK_POSTFLAGS);
