@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2012, 2014-2015 Free Software Foundation, Inc.
+   Copyright (C) 2010-2012, 2014-2016 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,17 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
-#if defined(HAVE_CONFIG_H)
-# include <config.h>
-#endif
-#include <stdlib.h>
-#include <string.h>
-#include <signal.h>
-#include <mailutils/mailutils.h>
-#include <mailutils/tls.h>
-#include "mailutils/libargp.h"
-#include "muaux.h"
 #include "mu.h"
+#include <signal.h>
+#include "muaux.h"
 
 #ifdef WITH_READLINE
 # include <readline/readline.h>
@@ -234,7 +226,7 @@ shell_help (int argc, char **argv)
 	print_comtab (mu_strout, com);
       else
 	{
-	  mu_printf ("No commands match `%s'.  Possibilties are:\n", name);
+	  mu_printf ("No commands match `%s'.  Possibilities are:\n", name);
 	  list_commands (shell_comtab, name);
 	}
     }
@@ -252,12 +244,16 @@ shell_help (int argc, char **argv)
 static int
 shell_prompt (int argc, char **argv)
 {
-  size_t size;
+  mu_wordsplit_t ws;
   
-  free (mutool_shell_prompt);
-  size = strlen (argv[1]);
-  mutool_shell_prompt = mu_alloc (size + 1);
-  mu_wordsplit_c_unquote_copy (mutool_shell_prompt, argv[1], size);
+  if (mu_wordsplit (argv[1], &ws, MU_WRDSF_NOSPLIT | MU_WRDSF_DEFFLAGS))
+    mu_error ("mu_wordsplit: %s", mu_wordsplit_strerror (&ws));
+  else
+    {
+      free (mutool_shell_prompt);
+      mutool_shell_prompt = mu_strdup (ws.ws_wordv[0]);
+    }
+  mu_wordsplit_free (&ws);
   return 0;
 }
 
@@ -293,11 +289,11 @@ get_history_file_name ()
       
       hname = mu_alloc (sizeof HISTFILE_PREFIX + strlen (rl_readline_name) +
 		        sizeof HISTFILE_SUFFIX - 1);
-      strcpy (hname, "~/.mu_");
+      strcpy (hname, HISTFILE_PREFIX);
       strcat (hname, rl_readline_name);
       strcat (hname, HISTFILE_SUFFIX);
       filename = mu_tilde_expansion (hname, MU_HIERARCHY_DELIMITER, NULL);
-      free(hname);
+      free (hname);
     }
   return filename;
 }
@@ -344,7 +340,7 @@ mutool_initialize_readline (const char *name)
 static void
 finish_readline ()
 {
-  write_history (get_history_file_name());
+  write_history (get_history_file_name ());
 }
 
 /* Attempt to complete on the contents of TEXT.  START and END bound the
@@ -521,7 +517,7 @@ execute_line (char *line)
   int status = 0;
   
   ws.ws_comment = "#";
-  ws.ws_escape = "\\\"";
+  ws.ws_escape[0] = ws.ws_escape[1] = "\\\\\"\"";
   rc = mu_wordsplit (line, &ws,
 		     MU_WRDSF_DEFFLAGS|MU_WRDSF_COMMENT|MU_WRDSF_ESCAPE|
 		     MU_WRDSF_INCREMENTAL|MU_WRDSF_APPEND);
